package com.mins01.jwschat;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class Room{
	
	static final Logger logger = LogManager.getLogger(Room.class.getName());
	
	public String roomName = "";
	public List<String> usersInRoom = new ArrayList<String>();
	Server server;
	HashMap<String, User>users;

	public Room(String roomName,Server server) {
		this.server = server;
		this.users = server.users;
		this.roomName = roomName;
		logger.info("방이름:"+this.roomName);
	}
	public synchronized  void join(User usr) {
		logger.info("입장 : "+roomName+":"+usr.uid);
		usersInRoom.add(usr.uid);
		
		//-- 조인 시스템 명령
		String stat = "system";
		ChatMsg cm = new ChatMsg();
		cm.put("r", this.roomName);
		cm.put("cmd", "joinRoom");
		cm.put("stat", stat);
		cm.put("data", this.roomName);
		cm.put("nick", "<"+stat+">");
		//broadcast(cm);
		usr.sendCm(cm);
		syncUserlist();
	
		this.broadcastMsg(roomName+"에 "+usr.nick+"님이 입장하셨습니다.","notice");
		
	}
	public void exit(User usr) {
		logger.info("퇴장 : "+roomName+":"+usr.uid);
		usersInRoom.remove(usr.uid);
		syncUserlist();
		this.broadcastMsg(roomName+"에 "+usr.nick+"님이 퇴장하셨습니다.","notice");
	}
	public synchronized List<String> getUserlist() {
		List<String> nicks = new ArrayList<String>();;
		for(String uid : this.usersInRoom){
			nicks.add(users.get(uid).nick);
		}
		Collections.sort(nicks);
		return nicks;
	}
	public synchronized  void syncUserlist() {
		String stat = "notice";
		ChatMsg cm = new ChatMsg();
		//cm.set(getUserlist(),"syncUserlist");
		cm.put("cmd","syncUserlist");
		cm.put("stat", stat);
		cm.put("data", getUserlist());
		cm.put("nick", "<"+stat+">");
		broadcast(cm);
	}
	public void broadcastMsg(String msg,String stat) {
		ChatMsg cm = new ChatMsg();
		cm.put("r", this.roomName);
		cm.put("stat", stat);
		cm.put("data", msg);
		cm.put("nick", "<"+stat+">");
		broadcast(cm);
	}
	public void broadcast(ChatMsg cm) {
		User usr;
		cm.put("r", this.roomName);
		synchronized ( usersInRoom ) {
			for(int i=usersInRoom.size()-1,m=0;i>=m;i--){
				if((usr = users.get(usersInRoom.get(i))) == null){
					logger.warn(usersInRoom.get(i)+":잘못된 유저 삭제");
					usersInRoom.remove(i); //잘못된 유저는 삭제한다.
				}else{
					usr.sendCm(cm);
				}
			}
		}
		logger.info("broadcast : "+usersInRoom.size()+":"+cm.toString());
	}
	//===== cmd action
	public void chat(ChatMsg req) {
		broadcast(req);		
	}
}
