package com.mins01.jwschat;

import java.io.Serializable;
import java.net.InetSocketAddress;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map.Entry;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;


public class Server  extends WebSocketServer implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	static final Logger logger = LogManager.getLogger(Server.class.getName());

	Integer unum = 0;
	HashMap <String,User> users = new HashMap<String , User>(); 
	Rooms rooms;
	Room lobby;
	
    public static void main( String[] args )
    {
    	//java.lang.System.setProperty("java.net.preferIPv6Addresses", "false");
    	//java.lang.System.setProperty("java.net.preferIPv4Stack", "true");
    	
    	Server sr = new Server(new InetSocketAddress(6001));
    	sr.start();
    }
	
    /**
     * 생성자
     * @param address
     */
	public Server(InetSocketAddress address) {
		super(address);
		logger.info("Server : "+address.getHostName()+":"+address.getPort());
        rooms = new Rooms(this);
        //-- 기본 방을 만든다.
        lobby = rooms.createRoom("lobby");
        rooms.createRoom("room1");
        rooms.createRoom("room2");
        rooms.createRoom("room3");
        
		// TODO Auto-generated constructor stub
	}

	/**
	 * 접속 종료시
	 */
	@Override
	public void onClose(WebSocket ws, int arg1, String arg2, boolean arg3) {
		// TODO Auto-generated method stub
		logger.info("onClose"+ws.toString()+":"+arg1);
		User usr = users.get(ws.toString());
		usr.exit();
		users.remove(usr.uid);
		usr = null;
		logger.info("총 접속인원 : "+users.size());
	}

	/**
	 * 에러 발생시
	 */
	@Override
	public void onError(WebSocket ws, Exception e) {
		// TODO Auto-generated method stub
		e.printStackTrace();
		logger.info("onError"+ws.toString()+":"+e.toString());
	}

	/**
	 * 메세지를 받았을 때
	 */
	@Override
	public  void onMessage(WebSocket ws, String raw) {
		// TODO Auto-generated method stub
		logger.info("onMessage"+ws.toString()+":"+raw);
		User usr = users.get(ws.toString());
		if(usr==null){
			logger.info("해당 user를 찾을 수 없음"+ws.toString()+":"+raw);
		}else{
			actionOnMessage(usr,raw);
		}
		//lobby.broadcast(raw);
		//sendToAll(msg);
	}
	
	/**
	 * 사용자 접속
	 */
	@Override
	public void onOpen(WebSocket ws, ClientHandshake ch) {
		// TODO Auto-generated method stub
		logger.info("onOpen"+ws.toString()+":"+ch.toString());
		String uid = ws.toString();
		//String nick = "USER"+String.format("%05d", (unum++));
		String nick = getRandNick(); //랜덤 닉네임
		User usr = new User(ws,uid,nick,this);
		
		//usr.sendMsg(usr.nick+"님 방문을 환영합니다");
		users.put(usr.uid, usr);
		logger.info("접속 : "+usr.uid+":"+usr.nick);
		logger.info("총 접속인원 : "+users.size());
		//usr.joinRoom(lobby);
		//usr.joinRoom(rooms.getRoom("room1"));
	}
	public String getRandNick() {
		String nick=""; 
		nick = "U"+String.format("%07d", (int)(Math.random()*9999999));
		while(whoByNick(nick)!=null){
			nick = "U"+String.format("%07d", (int)(Math.random()*9999999));
		}
		return nick;
	}
	public User whoByNick(String nick) {
		//--- 닉네임으로 사람 찾기 (Users 를 만들어야겠지만..)
		for(Entry<String, User> e : users.entrySet()) {
			if(e.getValue().nick.equals(nick)){
				return e.getValue();
			}
		}
		return null;
	}
	/**
	 * Sends text to all currently wsected WebSocket clients.
	 * 
	 * @param text
	 *            The String to send across the network.
	 * @throws InterruptedException
	 *             When socket related I/O errors occur.
	 */
	public void sendToAll( String text ) {
		Collection<WebSocket> wss = connections();
		synchronized ( wss ) {
			for( WebSocket c : wss ) {
				c.send( text );
			}
		}
	}
	
	/**
	 * commnad
	 */
	public synchronized  void actionOnMessage(User usr,String raw) {
		ChatMsg req = new ChatMsg(raw);
		usr.actionOnMessage(req);
	}
	public String[] decodeJson(String rawJson) {
		JSONObject data = (JSONObject) JSONValue.parse(rawJson);
		String roomName = (String) data.get("r");
		String msg = (String) data.get("msg");
		String[] r = {roomName,msg};
		return r;
	}
}
