package com.mins01.jwschat;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.java_websocket.WebSocket;

/**
 * User
 *
 */
public class User{
	static final Logger logger = LogManager.getLogger(User.class.getName());
	
	WebSocket ws;
	
	String uid = "";
	String nick = "";
	
	Server server = null;
	//List <Room> joinedRooms = new ArrayList<Room>();  
	//Map <String,Room> joinedRooms = new Hsa<Room>();
	Map<String, Room> joinedRooms = new HashMap<String, Room>();
	
	public User(WebSocket ws, String uid, String nick, Server server) {
		this.ws = ws;
		this.uid = uid;
		//this.nick = nick;
		this.server = server;
		setNick(nick);
		
	}
	public boolean sendCm(ChatMsg cm) {
		logger.info("응답:"+cm.toString());
    ws.send(cm.toString());
		return true;
	}
	public boolean sendMsg(String msg) {
		return this.sendMsg(msg,"","");
	}
	public boolean sendMsg(String msg,String stat,String cmd) {
		ChatMsg cm = new ChatMsg();
		cm.setData(msg);
		cm.put("data", msg);
		if(stat != ""){
			cm.put("cmd", "alert");
			cm.put("stat", stat);
			cm.put("nick", "<"+stat+">");
		}
		return sendCm(cm);
	}
	public synchronized  void joinRoom(String roomName) {
		//joinedRooms.add(room);
		if(joinedRooms.get(roomName)!=null){
			sendMsg("방 ["+roomName+"]에 이미 참여하고 계십니다.","error","alert");
			return;
		}
		Room room = server.rooms.getRoom(roomName);
		if(room==null){
			sendMsg("방 ["+roomName+"]이 없습니다.","error","alert");
			return;
		}
		this.joinRoom(room);
	}
	public synchronized  void joinRoom(Room room) {
		//joinedRooms.add(room);
		joinedRooms.put(room.roomName,room);
		this.syncJoinedRoomlist();
		room.join(this);
	}
	public void exitRoom(String roomName) {
		//joinedRooms.add(room);
		if(joinedRooms.get(roomName)==null){
			sendMsg("방 ["+roomName+"]에 이미 참여하고있지 않습니다","error","alert");
			return;
		}
		Room room = server.rooms.getRoom(roomName);
		if(room==null){
			sendMsg("방 ["+roomName+"]이 없습니다.","error","alert");
			return;
		}
		this.exitRoom(room);
	}	
	public void exitRoom(Room room) {
		joinedRooms.remove(room.roomName);
		room.exit(this);
	}
	public synchronized  void exit() {
		for(Entry<String, Room> e : joinedRooms.entrySet()) {
        //exitRoom(e.getValue());
				//joinedRooms.remove(e.getKey());
			e.getValue().exit(this);
    }
		logger.info(this.nick+": 사용자 퇴장(exit");
	}
	public synchronized  void setNick(String nick) {
		this.nick = nick;
		//sendMsg("["+this.nick+"]로 대화명이 변경되었습니다.","notice","alert");
		ChatMsg cm = new ChatMsg();
		cm.put("cmd", "setNick");
		cm.put("data", nick);
		this.sendCm(cm);
		return;		
	}
	public synchronized  void changeNick(String nick) {
		if(this.nick.equals(nick)){
			sendMsg("같은 대화명으로 변경할 수 없습니다.","error","alert");
			return;
		}
		if(nick.length()>10){
			sendMsg("너무 긴 대화명입니다.","error","alert");
			return;
		}
		//--- 중복 닉네임 검사
		User usr2 = server.whoByNick(nick);
		if(usr2 == null){
			//없음
		}else if(usr2.uid.equals(this.uid)){
			//본인
		}else{
			sendMsg("이미 사용중인 대화명입니다.","error","alert");
			setNick(this.nick);
			return;
		}
		
		//---
		String msg = "["+this.nick +"]님이 ["+nick+"]로 대화명을 변경하셨습니다.";
		String stat = "notice";
		setNick(nick);
		for(Entry<String, Room> e : joinedRooms.entrySet()) {
			e.getValue().broadcastMsg(msg, stat);;
			e.getValue().syncUserlist();
		}
		sendMsg("["+nick+"]로 대화명을 변경하셨습니다.","error","alert");
		return;		
	}
	public synchronized  void actionOnMessage(ChatMsg req) {
		logger.info(this.nick+"가 req를 하였습니다."+req);
		req.put("nick", this.nick);
		String cmd = (String) (req.get("cmd")==null?"":req.get("cmd"));
		String r = (String) req.get("r");
		//String data = (String)  req.get("data");
		if(r!=null){ //방 안에서
			Room room = joinedRooms.get(r);
			if(room == null){
				logger.error(r+": 이라는 방이 있지 않습니다");
				return;
			}
			try{
				switch(cmd){
					case "chat" :;
					case "null" :;
					case "" : room.chat(req); break;
				}
			}catch(Exception e){
				
			}
			//room.broadcast(cm);
		}else{ //전역 명령어
			switch(cmd){
				case "syncRoomlist":
					server.rooms.syncRoomlist(this);
					break;
				case "syncJoinedRoomlist":
					this.syncJoinedRoomlist();
					break;
				case "joinRoom":
					this.joinRoom((String) req.get("data"));
					break;
				case "exitRoom":
					this.exitRoom((String) req.get("data"));
					break;
				case "changeNick":
					this.changeNick((String) req.get("data"));
					break;					
			}
		}
		
	}
	public void syncJoinedRoomlist() {
		List<String> list = new ArrayList<String>(joinedRooms.keySet());
		Collections.sort(list);
		ChatMsg cm = new ChatMsg();
		cm.put("cmd", "syncJoinedRoomlist");
		cm.put("data", list);
		this.sendCm(cm);
	}
}
